/*
 * $Id: jstring.h,v 1.37 2000/05/18 14:28:14 resort Exp $
 *
 * This work is copyright James Hawtin (oolon@ankh.org)
 * 2000/1/3
 *
 * This product contains no warranty, liability or fitness to use. The author
 * accepts no liability for problems caused by direct or indirect use of this
 * work. By using this work you agree to all the conditions of the copyright.
 *
 * The ownership of this work will remain that of the author. However 
 * permission is given to use this work in any way required so long as the
 * above copyright message is left intact.
 *
 * The author reserves the right to use any changes made by him or submitted
 * by others in any future release.
 */

#ifndef __J_STRING_H__
#define __J_STRING_H__

#include <jtools.h>
#include <jdata.h>
#include <iostream>
#include <fcntl.h>

const size_t Jstring_prefered_size(512);

class Jstring
{
 public:
  typedef Jstring&(*__jconfunc)(Jstring &);
  Jstring();
  Jstring(const char *);
  Jstring(const Jstring &);
  
  const Jstring &operator+=(const Jstring &rhs)  { *this << rhs; return *this; };
  Jstring &operator<<(const Jstring &);
  Jstring &operator>>(const Jstring &);
  
  const Jstring &operator+=(const char val) { *this << val; return *this; }; 
  const Jstring &operator+=(int val) { *this << val; return *this; };

  Jstring &operator<<(const char);
  Jstring &operator>>(const char);

#ifdef LONGLONG
  Jstring &operator<<(const long long);
  Jstring &operator>>(const long long);
  
  Jstring &operator<<(const unsigned long long);
  Jstring &operator>>(const unsigned long long);
#endif

  Jstring &operator<<(const int);
  Jstring &operator>>(const int);

  Jstring &operator<<(const unsigned int);
  Jstring &operator>>(const unsigned int);

  Jstring &operator<<(const short);
  Jstring &operator>>(const short);

  Jstring &operator<<(const unsigned short);
  Jstring &operator>>(const unsigned short);

  Jstring &operator<<(const long);
  Jstring &operator>>(const long);
  
  Jstring &operator<<(const unsigned long);
  Jstring &operator>>(const unsigned long);
  
  Jstring &operator>>(const char *);
  Jstring &operator<<(const char *);

  // Support functions like endl;
  
  Jstring &operator<<(__jconfunc func) { (*func)(*this); return *this; };
  Jstring &operator>>(__jconfunc func) { (*func)(*this); return *this; };

  Jstring &operator=(const Jstring &);
  Jstring &operator=(const char *);

  const Jstring operator+(const Jstring &val);
  ~Jstring();

  // Get the size of the string
  size_t len(void) const;

  // get the whole value as a char array
  const char * value(void) const;

  // get a single character in the string
  char get(size_t) const;

  // set a single character in the string
  void set(size_t, char c);

#ifdef __REF_COUNT__
  int ref;
  static int total;
#endif
  void debugprint(char *title=0) const;
  int load(const char *);
  int load(Jstring &tmp) { return load(tmp.value()); };
  int save(const char *, int options=O_CREAT|O_TRUNC, int mode=0644);
  int save(Jstring &tmp, int options=O_CREAT|O_TRUNC, int mode=0644)
    { return save(tmp.value(), options, mode); };
  void trunc(size_t newsize);
  void chomp(char c=' ', int bytes=1);
  void chop(int bytes=1) { if (real) trunc(len() - bytes);};
  void rtrunc(size_t newsize);
  void rchomp(char c=' ', int bytes=1);
  void rchop(int bytes=1) { if (real) rtrunc(len() - bytes);};
  void rep_chr(char c, int rep);
  Jstring strchr(const char c);
  Jstring strrchr(const char c);
  char get_char(void);
  char rget_char(void);
  Jstring get_word(int strip=false);
  Jstring get_line(int newline=false);
  void strip_white(void);
  void strip_end_white(void);
  void clear_string(int forcefree=false);
  int strcmp(const char *str);
  int strcasecmp(const char *str);
  int strcmp(const Jstring &str);
  int strcasecmp(const Jstring &str);
  void lowercase(void);
  int cmp(const Jstring &str);
  int casecmp(const Jstring &str);
  int cmp(const char *str);
  int casecmp(const char *str);
  int operator==(const Jstring &tmp) { return this->cmp(tmp); };
  int operator==(const char *str) { return this->cmp(str); };
 private:
  static size_t prefered_size;
  void delete_jdata(void)
    { if ((real) && (real->remove()<1)) Jdata::destroy(real); real=0; offset=0; };
  void add_num_end(const int, Jstring &);
  void add_num_begin(const int, Jstring &);
  void shift(size_t move);
  void resize_plus(const int, const int=true);
  int jstrlen(const char *const);
  friend Jstring &tag_start(Jstring &val);
  friend Jstring &tag_end(Jstring &val);
  Jdata *real;
  size_t offset;
};

extern ostream &operator<<(ostream &stream, const Jstring &string);
extern Jstring &endl(Jstring &val);
inline Jstring &jstr_clear(Jstring &val) { val.clear_string(); return val; };
Jstring left(Jstring &ref, int, char pad=' ');
Jstring right(Jstring &, int, char pad=' ');
Jstring centre(Jstring &, int, int right=false, char pad=' ');
inline Jstring left(const char *ref, int a, char pad=' ') { Jstring tmp(ref); return left(tmp, a, pad); };
inline Jstring right(const char *ref, int a, char pad=' ') { Jstring tmp(ref); return right(tmp, a, pad);};
inline Jstring centre(const char *ref, int a, int r = false, char pad=' ') { Jstring tmp(ref); return centre(tmp, a, r, pad);};
inline Jstring left(const int ref, int a, char pad=' ') { Jstring tmp; tmp << ref; return left(tmp, a, pad); };
inline Jstring right(const int ref, int a, char pad=' ') { Jstring tmp; tmp << ref; return right(tmp, a, pad);};
inline Jstring centre(const int ref, int a, int r = false, char pad=' ') { Jstring tmp; tmp << ref; return centre(tmp, a, r, pad);};

#endif
